// mixed.cpp
// by David Lambert - 2013-02-11

#include <iostream>
#include "mixed.h"
using namespace std;

Mixed::Mixed(int w)
{
	whole = w;
	numerator = 0;
	denominator = 1;
}

Mixed::Mixed (int w, int n, int d)
{
	if (!Set(w,n,d))
	{
		whole = 0;
		numerator = 0;
		denominator = 1;
	}
}

bool Mixed::Set (int w, int n = 0, int d = 1)
{
	if (d < 1)	//denominator better be positive
		return false;
	if (w < 0 && n < 0)	//both can't be negative
		return false;
	if (w > 0 && n < 0)	//only first non-zero can be negative
		return false;

	// they must be good if we've gotten this far, so set 'em!
	whole = w;
	numerator = n;
	denominator = d;
	return true;
}

double Mixed::Evaluate() const
{
	if (whole < 0)
		return whole - (double(numerator) / denominator);

	return whole + (double(numerator) / denominator);
}

void Mixed::Simplify()
{
	bool neg = false;
	int k, t1 = 0, t2 = 0;

	if (numerator < 0)	//make numerator positive for calculations
	{
		numerator *= -1;
		neg = true;
	}

	if (whole < 0)		//make whole # positive for calculations
	{
		whole *= -1;
		neg = true;
	}

	if (numerator > denominator)	//convert improper fraction to mixed number
	{
		t1 = numerator / denominator;
		t2 = t1 * denominator;
		numerator = numerator - t2;
		whole += t1;
	}
	
	if (numerator == denominator)	
		numerator, denominator = 1;	//n and d are the same, so reduce it to 1
	else									//if not, reduce it
	{
		if (numerator < denominator)
			k = denominator;
		else
			k = numerator;
		for (int i = k; i > 0; i--)	//find gcf
		{
			if(numerator % i == 0 && denominator % i == 0)
			{
				numerator /= i;
				denominator /= i;
			}
		}
	}

	if (neg)	//make back into neg, if was neg to start
	{
		if (whole == 0)
			numerator *= -1;
		else if (whole > 0)
			whole *= -1;
	}
}

void Mixed::ToFraction()
{
	bool neg = false;

	if (whole != 0) // if no whole #, it's already a fraction!
	{
		if (whole < 0)		//make whole # positive for calculations
		{
			whole *= -1;
			neg = true;
		}
		
		numerator = numerator + (whole * denominator);	//convert to fraction
		whole = 0;	//don't need this anymore
	
		if (neg)	//make back into neg, if was neg to start
			numerator *= -1;
	}
}

ostream& operator<< (ostream& s, const Mixed& m)
{
	if (m.whole == 0 && m.numerator == 0)	//it actually is zero
		s << m.whole;
	else if (m.whole == 0)		//only a fraction
		s << m.numerator << '/' << m.denominator;
	else if (m.numerator == 0)		//only a whole number
		s << m.whole;
	else						//must be a mixed number
		s << m.whole << ' ' << m.numerator << '/' << m.denominator;

  return s;
}

istream& operator>> (istream& s, Mixed& m)
{
	char blackhole;	//for eating the slash
	int w,n,d;
	s >> w >> n >> blackhole >> d;
	
	if (!m.Set(w,n,d))	//if input no good, set to default
	{
		m.whole = 0;
		m.numerator = 0;
		m.denominator = 1;
	}
	return s;
}

Mixed operator+ (const Mixed& m1, const Mixed& m2)
{
	Mixed r,t1,t2;

	t1 = m1;		// make copies of input Mixed's to do calculations on them
	t2 = m2;
	t1.ToFraction();	// convert 'em to improper fractions for easier calculation
	t2.ToFraction();
	r.numerator = (t1.numerator * t2.denominator) + (t2.numerator * t1.denominator);
	r.denominator = t1.denominator * t2.denominator;
	r.Simplify();	//convert back to mixed #

	return r;
}

Mixed operator- (const Mixed& m1, const Mixed& m2)
{
	return m1 + (m2 * -1);	//uses overloaded "+" and "*" operators
}

Mixed operator* (const Mixed& m1, const Mixed& m2)
{
	Mixed r,t1,t2;

	t1 = m1;		// make copies of input Mixed's to do calculations on them
	t2 = m2;
	t1.ToFraction();	// convert 'em to improper fractions for easier calculation
	t2.ToFraction();
	r.numerator = t1.numerator * t2.numerator;	//do the calcs
	r.denominator = t1.denominator * t2.denominator;
	r.Simplify();	//convert back to mixed #

	return r;
}

Mixed operator/ (const Mixed& m1, const Mixed& m2)
{
	Mixed r,t1,t2;

	t1 = m1;		// make copies of input Mixed's to do calculations on them
	t2 = m2;
	if (t2 == 0)	//if second operand is 0, return 0 (using overloaded "==")
		return t2;
	t1.ToFraction();	// convert 'em to improper fractions for easier calculation
	t2.ToFraction();
	r.numerator = t1.numerator * t2.denominator;	//do the calcs
	r.denominator = t1.denominator * t2.numerator;
	r.Simplify();	//convert back to mixed #

	return r;
}

Mixed operator++ (Mixed& m1)	//++Mixed
{
	m1.ToFraction();
	m1 = m1 + 1;
	m1.Simplify();

	return m1;
}

Mixed operator++ (Mixed& m1, int dummy)	//Mixed++
{
	Mixed temp;
	temp = m1;
	m1 = m1 + 1;
	temp.Simplify();

	return temp;
}

Mixed operator-- (Mixed& m1)	//--Mixed
{
	m1.ToFraction();
	m1 = m1 - 1;
	m1.Simplify();

	return m1;
}

Mixed operator-- (Mixed& m1, int dummy)	//Mixed--
{
	Mixed temp;
	temp = m1;
	m1 = m1 - 1;
	temp.Simplify();

	return temp;
}

bool operator< (const Mixed& m1, const Mixed& m2)
{
	Mixed t1 = m1, t2 = m2; //make copies, leave the originals alone
	t1.ToFraction();	//convert to impropers for easier calculations
	t2.ToFraction();

	t1.denominator = t1.denominator * t2.denominator; //find LCD
	t2.denominator = t1.denominator;		//make both d's the same
	t1.numerator *= (t1.denominator / m1.denominator);	//calculate n's
	t2.numerator *= (t2.denominator / m2.denominator);
	
	if (t1.numerator < t2.numerator)	//do the test
		return true;
	else
		return false;
}

bool operator== (const Mixed& m1, const Mixed& m2)
{
	if (!(m1 < m2) && !(m1 > m2))
		return true;
	else
		return false;
}

bool operator> (const Mixed& m1, const Mixed& m2)
{
	if (m2 < m1)
		return true;
	else
		return false;
}

bool operator!= (const Mixed& m1, const Mixed& m2)
{
	if (m1 < m2 || m1 > m2)
		return true;
	else
		return false;
}

bool operator<= (const Mixed& m1, const Mixed& m2)
{
	if ((m1 < m2) || (m1 == m2))
		return true;
	else
		return false;
}

bool operator>= (const Mixed& m1, const Mixed& m2)
{
	if ((m1 > m2) || (m1 == m2))
		return true;
	else
		return false;
}
